@@ -74,6 +74,8 @@ enum
 #define DEFAULT_CHECK_IMPERFECT_OFFSET    FALSE
 #define DEFAULT_SIGNAL_HANDOFFS           TRUE
 #define DEFAULT_TS_OFFSET               0
+#define DEFAULT_DROP_ALLOCATION         FALSE
+#define DEFAULT_EOS_AFTER               -1
 
 enum
 {
@@ -91,7 +93,9 @@ enum
   PROP_TS_OFFSET,
   PROP_CHECK_IMPERFECT_TIMESTAMP,
   PROP_CHECK_IMPERFECT_OFFSET,
-  PROP_SIGNAL_HANDOFFS
+  PROP_SIGNAL_HANDOFFS,
+  PROP_DROP_ALLOCATION,
+  PROP_EOS_AFTER
 };
 
 
@@ -109,6 +113,8 @@ static void gst_identity_get_property (GObject * object, guint prop_id,
 
 static gboolean gst_identity_sink_event (GstBaseTransform * trans,
     GstEvent * event);
+static gboolean gst_identity_src_event (GstBaseTransform * trans,
+    GstEvent * event);
 static GstFlowReturn gst_identity_transform_ip (GstBaseTransform * trans,
     GstBuffer * buf);
 static gboolean gst_identity_start (GstBaseTransform * trans);
@@ -119,6 +125,7 @@ static gboolean gst_identity_accept_caps (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps);
 static gboolean gst_identity_query (GstBaseTransform * base,
     GstPadDirection direction, GstQuery * query);
+static GstClock *gst_identity_provide_clock (GstElement * element);
 
 static guint gst_identity_signals[LAST_SIGNAL] = { 0 };
 
@@ -157,7 +164,7 @@ gst_identity_class_init (GstIdentityClass * klass)
           DEFAULT_SLEEP_TIME, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_ERROR_AFTER,
       g_param_spec_int ("error-after", "Error After", "Error after N buffers",
-          G_MININT, G_MAXINT, DEFAULT_ERROR_AFTER,
+          -1, G_MAXINT, DEFAULT_ERROR_AFTER,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_DROP_PROBABILITY,
       g_param_spec_float ("drop-probability", "Drop Probability",
@@ -222,14 +229,31 @@ gst_identity_class_init (GstIdentityClass * klass)
   /**
    * GstIdentity:signal-handoffs
    *
-   * If set to #TRUE, the identity will emit a handoff signal when handling a buffer.
-   * When set to #FALSE, no signal will be emitted, which might improve performance.
+   * If set to %TRUE, the identity will emit a handoff signal when handling a buffer.
+   * When set to %FALSE, no signal will be emitted, which might improve performance.
    */
   g_object_class_install_property (gobject_class, PROP_SIGNAL_HANDOFFS,
       g_param_spec_boolean ("signal-handoffs",
           "Signal handoffs", "Send a signal before pushing the buffer",
           DEFAULT_SIGNAL_HANDOFFS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_DROP_ALLOCATION,
+      g_param_spec_boolean ("drop-allocation", "Drop allocation query",
+          "Don't forward allocation queries", DEFAULT_DROP_ALLOCATION,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstIdentity:eos-after
+   *
+   * EOS after N buffers.
+   *
+   * Since: 1.16
+   **/
+  g_object_class_install_property (gobject_class, PROP_EOS_AFTER,
+      g_param_spec_int ("eos-after", "EOS After", "EOS after N buffers",
+          -1, G_MAXINT, DEFAULT_EOS_AFTER,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GstIdentity::handoff:
    * @identity: the identity instance
@@ -241,8 +265,7 @@ gst_identity_class_init (GstIdentityClass * klass)
   gst_identity_signals[SIGNAL_HANDOFF] =
       g_signal_new ("handoff", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstIdentityClass, handoff), NULL, NULL,
-      g_cclosure_marshal_generic, G_TYPE_NONE, 1,
-      GST_TYPE_BUFFER | G_SIGNAL_TYPE_STATIC_SCOPE);
+      NULL, G_TYPE_NONE, 1, GST_TYPE_BUFFER | G_SIGNAL_TYPE_STATIC_SCOPE);
 
   gobject_class->finalize = gst_identity_finalize;
 
@@ -255,8 +278,11 @@ gst_identity_class_init (GstIdentityClass * klass)
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_identity_change_state);
+  gstelement_class->provide_clock =
+      GST_DEBUG_FUNCPTR (gst_identity_provide_clock);
 
   gstbasetrans_class->sink_event = GST_DEBUG_FUNCPTR (gst_identity_sink_event);
+  gstbasetrans_class->src_event = GST_DEBUG_FUNCPTR (gst_identity_src_event);
   gstbasetrans_class->transform_ip =
       GST_DEBUG_FUNCPTR (gst_identity_transform_ip);
   gstbasetrans_class->start = GST_DEBUG_FUNCPTR (gst_identity_start);
@@ -271,6 +297,7 @@ gst_identity_init (GstIdentity * identity)
 {
   identity->sleep_time = DEFAULT_SLEEP_TIME;
   identity->error_after = DEFAULT_ERROR_AFTER;
+  identity->error_after_counter = DEFAULT_ERROR_AFTER;
   identity->drop_probability = DEFAULT_DROP_PROBABILITY;
   identity->drop_buffer_flags = DEFAULT_DROP_BUFFER_FLAGS;
   identity->datarate = DEFAULT_DATARATE;
@@ -284,8 +311,13 @@ gst_identity_init (GstIdentity * identity)
   identity->signal_handoffs = DEFAULT_SIGNAL_HANDOFFS;
   identity->ts_offset = DEFAULT_TS_OFFSET;
   g_cond_init (&identity->blocked_cond);
+  identity->eos_after = DEFAULT_EOS_AFTER;
+  identity->eos_after_counter = DEFAULT_EOS_AFTER;
 
   gst_base_transform_set_gap_aware (GST_BASE_TRANSFORM_CAST (identity), TRUE);
+
+  GST_OBJECT_FLAG_SET (identity, GST_ELEMENT_FLAG_PROVIDE_CLOCK);
+  GST_OBJECT_FLAG_SET (identity, GST_ELEMENT_FLAG_REQUIRE_CLOCK);
 }
 
 static void
@@ -310,7 +342,7 @@ gst_identity_do_sync (GstIdentity * identity, GstClockTime running_time)
       return GST_FLOW_FLUSHING;
     }
 
-    while (identity->blocked)
+    while (identity->blocked && !identity->flushing)
       g_cond_wait (&identity->blocked_cond, GST_OBJECT_GET_LOCK (identity));
 
     if (identity->flushing) {
@@ -397,7 +429,14 @@ gst_identity_sink_event (GstBaseTransform * trans, GstEvent * event)
 
       /* This is the first segment, send out a (0, -1) segment */
       gst_segment_init (&segment, segment.format);
+      if (identity->seek_segment.format != GST_FORMAT_UNDEFINED) {
+        segment.time = identity->seek_segment.time;
+        segment.base = identity->seek_segment.base;
+        gst_segment_init (&identity->seek_segment, GST_FORMAT_UNDEFINED);
+      }
+
       news = gst_event_new_segment (&segment);
+      GST_EVENT_SEQNUM (news) = GST_EVENT_SEQNUM (event);
 
       gst_pad_event_default (trans->sinkpad, GST_OBJECT_CAST (trans), news);
     } else {
@@ -436,27 +475,73 @@ gst_identity_sink_event (GstBaseTransform * trans, GstEvent * event)
     /* eat up segments */
     gst_event_unref (event);
     ret = TRUE;
-  } else {
-    if (GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_START) {
+    goto done;
+  }
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FLUSH_START:
       GST_OBJECT_LOCK (identity);
       identity->flushing = TRUE;
+      g_cond_signal (&identity->blocked_cond);
       if (identity->clock_id) {
         GST_DEBUG_OBJECT (identity, "unlock clock wait");
         gst_clock_id_unschedule (identity->clock_id);
       }
       GST_OBJECT_UNLOCK (identity);
-    } else if (GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_STOP) {
+      break;
+    case GST_EVENT_FLUSH_STOP:
       GST_OBJECT_LOCK (identity);
       identity->flushing = FALSE;
+      trans->have_segment = FALSE;
       GST_OBJECT_UNLOCK (identity);
-    }
-
-    ret = GST_BASE_TRANSFORM_CLASS (parent_class)->sink_event (trans, event);
+      break;
+    default:
+      break;
   }
 
+  ret = GST_BASE_TRANSFORM_CLASS (parent_class)->sink_event (trans, event);
+
+done:
   return ret;
 }
 
+static gboolean
+gst_identity_src_event (GstBaseTransform * trans, GstEvent * event)
+{
+  GstIdentity *identity = GST_IDENTITY (trans);
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_SEEK:
+    {
+      gdouble rate;
+      GstFormat fmt;
+      GstSeekFlags flags;
+      GstSeekType start_type, stop_type;
+      gint64 start, stop;
+      gst_event_parse_seek (event, &rate, &fmt, &flags, &start_type,
+          &start, &stop_type, &stop);
+
+      GST_OBJECT_LOCK (identity);
+      gst_segment_init (&identity->seek_segment, fmt);
+      if (!gst_segment_do_seek (&identity->seek_segment, rate, fmt,
+              flags, start_type, start, stop_type, stop, NULL)) {
+        GST_WARNING_OBJECT (identity, "Could not run seek %" GST_PTR_FORMAT,
+            event);
+        GST_OBJECT_UNLOCK (identity);
+
+        return FALSE;
+      }
+      GST_OBJECT_UNLOCK (identity);
+
+      break;
+    }
+    default:
+      break;
+  }
+
+  return GST_BASE_TRANSFORM_CLASS (parent_class)->src_event (trans, event);
+}
+
 static void
 gst_identity_check_imperfect_timestamp (GstIdentity * identity, GstBuffer * buf)
 {
@@ -588,6 +673,7 @@ gst_identity_update_last_message_for_buffer (GstIdentity * identity,
       GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET_END (buf),
       GST_BUFFER_FLAGS (buf), flag_str, meta_str ? meta_str : "none", buf);
   g_free (flag_str);
+  g_free (meta_str);
 
   GST_OBJECT_UNLOCK (identity);
 
@@ -617,12 +703,18 @@ gst_identity_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
   identity->prev_offset_end = GST_BUFFER_OFFSET_END (buf);
   identity->prev_offset = GST_BUFFER_OFFSET (buf);
 
-  if (identity->error_after >= 0) {
-    identity->error_after--;
-    if (identity->error_after == 0)
+  if (identity->error_after_counter >= 0) {
+    identity->error_after_counter--;
+    if (identity->error_after_counter == 0)
       goto error_after;
   }
 
+  if (identity->eos_after_counter >= 0) {
+    identity->eos_after_counter--;
+    if (identity->eos_after_counter == 0)
+      goto eos_after;
+  }
+
   if (identity->drop_probability > 0.0) {
     if ((gfloat) (1.0 * rand () / (RAND_MAX)) < identity->drop_probability)
       goto dropped;
@@ -656,10 +748,21 @@ gst_identity_transform_ip (GstBaseTransform * trans, GstBuffer * buf)
     g_signal_emit (identity, gst_identity_signals[SIGNAL_HANDOFF], 0, buf);
 
   if (trans->segment.format == GST_FORMAT_TIME) {
-    rundts = gst_segment_to_running_time (&trans->segment,
-        GST_FORMAT_TIME, GST_BUFFER_DTS (buf));
-    runpts = gst_segment_to_running_time (&trans->segment,
-        GST_FORMAT_TIME, GST_BUFFER_PTS (buf));
+    if (trans->segment.rate > 0) {
+      runpts = gst_segment_to_running_time (&trans->segment,
+          GST_FORMAT_TIME, GST_BUFFER_PTS (buf));
+      rundts = gst_segment_to_running_time (&trans->segment,
+          GST_FORMAT_TIME, GST_BUFFER_DTS (buf));
+    } else {
+      runpts = gst_segment_to_running_time (&trans->segment,
+          GST_FORMAT_TIME, GST_CLOCK_TIME_IS_VALID (buf->duration)
+          && GST_CLOCK_TIME_IS_VALID (buf->pts) ? buf->pts +
+          buf->duration : buf->pts);
+      rundts = gst_segment_to_running_time (&trans->segment,
+          GST_FORMAT_TIME, GST_CLOCK_TIME_IS_VALID (buf->duration)
+          && GST_CLOCK_TIME_IS_VALID (buf->dts) ? buf->dts +
+          buf->duration : buf->dts);
+    }
   }
 
   if (GST_CLOCK_TIME_IS_VALID (rundts))
@@ -692,6 +795,11 @@ error_after:
         (_("Failed after iterations as requested.")), (NULL));
     return GST_FLOW_ERROR;
   }
+eos_after:
+  {
+    GST_DEBUG_OBJECT (identity, "EOS after iterations as requested.");
+    return GST_FLOW_EOS;
+  }
 dropped:
   {
     if (!identity->silent) {
@@ -759,6 +867,12 @@ gst_identity_set_property (GObject * object, guint prop_id,
     case PROP_SIGNAL_HANDOFFS:
       identity->signal_handoffs = g_value_get_boolean (value);
       break;
+    case PROP_DROP_ALLOCATION:
+      identity->drop_allocation = g_value_get_boolean (value);
+      break;
+    case PROP_EOS_AFTER:
+      identity->eos_after = g_value_get_int (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -811,7 +925,7 @@ gst_identity_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_boolean (value, identity->sync);
       break;
     case PROP_TS_OFFSET:
-      identity->ts_offset = g_value_get_int64 (value);
+      g_value_set_int64 (value, identity->ts_offset);
       break;
     case PROP_CHECK_IMPERFECT_TIMESTAMP:
       g_value_set_boolean (value, identity->check_imperfect_timestamp);
@@ -822,6 +936,12 @@ gst_identity_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_SIGNAL_HANDOFFS:
       g_value_set_boolean (value, identity->signal_handoffs);
       break;
+    case PROP_DROP_ALLOCATION:
+      g_value_set_boolean (value, identity->drop_allocation);
+      break;
+    case PROP_EOS_AFTER:
+      g_value_set_int (value, identity->eos_after);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -835,13 +955,27 @@ gst_identity_start (GstBaseTransform * trans)
 
   identity = GST_IDENTITY (trans);
 
+  if (identity->eos_after != DEFAULT_EOS_AFTER
+      && identity->error_after != DEFAULT_ERROR_AFTER)
+    goto both_afters_defined;
+
   identity->offset = 0;
   identity->prev_timestamp = GST_CLOCK_TIME_NONE;
   identity->prev_duration = GST_CLOCK_TIME_NONE;
   identity->prev_offset_end = GST_BUFFER_OFFSET_NONE;
   identity->prev_offset = GST_BUFFER_OFFSET_NONE;
+  identity->error_after_counter = identity->error_after;
+  identity->eos_after_counter = identity->eos_after;
 
   return TRUE;
+
+  /* ERROR */
+both_afters_defined:
+  {
+    GST_ELEMENT_ERROR (identity, CORE, FAILED,
+        (_("eos-after and error-after can't both be defined.")), (NULL));
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -887,6 +1021,12 @@ gst_identity_query (GstBaseTransform * base, GstPadDirection direction,
 
   identity = GST_IDENTITY (base);
 
+  if (GST_QUERY_TYPE (query) == GST_QUERY_ALLOCATION &&
+      identity->drop_allocation) {
+    GST_DEBUG_OBJECT (identity, "Dropping allocation query.");
+    return FALSE;
+  }
+
   ret = GST_BASE_TRANSFORM_CLASS (parent_class)->query (base, direction, query);
 
   if (GST_QUERY_TYPE (query) == GST_QUERY_LATENCY) {
@@ -909,8 +1049,12 @@ gst_identity_query (GstBaseTransform * base, GstPadDirection direction,
     GST_OBJECT_LOCK (identity);
     if (live)
       identity->upstream_latency = min;
-    else
+    else {
       identity->upstream_latency = 0;
+      /* if we interface a non-live source, then we know there is no
+       * limit in the maximum latency */
+      max = -1;
+    }
     GST_OBJECT_UNLOCK (identity);
 
     gst_query_set_latency (query, live || identity->sync, min, max);
@@ -940,7 +1084,7 @@ gst_identity_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       GST_OBJECT_LOCK (identity);
       identity->blocked = FALSE;
-      g_cond_broadcast (&identity->blocked_cond);
+      g_cond_signal (&identity->blocked_cond);
       GST_OBJECT_UNLOCK (identity);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
@@ -951,7 +1095,7 @@ gst_identity_change_state (GstElement * element, GstStateChange transition)
         gst_clock_id_unschedule (identity->clock_id);
       }
       identity->blocked = FALSE;
-      g_cond_broadcast (&identity->blocked_cond);
+      g_cond_signal (&identity->blocked_cond);
       GST_OBJECT_UNLOCK (identity);
       break;
     default:
@@ -982,3 +1126,10 @@ gst_identity_change_state (GstElement * element, GstStateChange transition)
 
   return ret;
 }
+
+/* FIXME: GStreamer 2.0 */
+static GstClock *
+gst_identity_provide_clock (GstElement * element)
+{
+  return gst_system_clock_obtain ();
+}
